from django.urls import reverse

from .logging import logger
from .registry import registry


def ajax_url(name: str):
    if name not in registry:
        logger.warning("view `%s` is not registered" % name)
    return reverse("ajax_views:router", kwargs={"name": name})



import json
from typing import Any

from django.template import Library
from django.urls import reverse
from django.utils.safestring import mark_safe

from .. import helpers
from ..registry import registry

try:
    import jinja2
except ImportError:
    jinja2 = None


register = Library()


def registry_to_json():
    data: dict[str, Any] = {}
    for name in registry:
        target = data
        path, _, key = name.rpartition(".")
        if path:
            for part in path.split("."):
                target = target.setdefault(part, {})
        target[key] = reverse("ajax_views:router", args=[name])
    return json.dumps(data)


@register.simple_tag(name="ajax_url")
def do_ajax_url(name):
    return helpers.ajax_url(name)


@register.simple_tag(name="ajax_views_json")
def ajax_views_json():
    return mark_safe(registry_to_json())


if jinja2 is not None:
    from jinja2 import nodes
    from jinja2.ext import Extension

    class AjaxViewsExtension(Extension):
        tags = {"ajax_views_json"}

        def parse(self, parser):
            lineno = next(parser.stream).lineno
            return nodes.CallBlock(
                self.call_method("_ajax_views_json"), [], [], []
            ).set_lineno(lineno)

        @staticmethod
        def _ajax_views_json(caller):
            return registry_to_json()

    # django-jinja support
    try:
        from django_jinja import library
    except ImportError:
        pass
    else:
        library.global_function(name="ajax_url")(do_ajax_url)
        library.extension(AjaxViewsExtension)



import django
from django.urls import ResolverMatch, URLPattern
from django.urls.resolvers import RegexPattern

from .registry import registry
from .views import router

if django.VERSION >= (4, 1):  # noqa

    class AjaxURLPattern(URLPattern):
        def resolve(self, path):
            match = self.pattern.match(path)
            if match:
                new_path, args, captured_kwargs = match
                # Pass any default args as **kwargs.
                kwargs = {**captured_kwargs, **self.default_args}

                name = kwargs.pop("name")
                if name not in registry:
                    return

                view = registry[name]
                return ResolverMatch(
                    view,
                    args,
                    kwargs,
                    self.pattern.name,
                    route=str(self.pattern),
                    captured_kwargs=captured_kwargs,
                    extra_kwargs=self.default_args,
                )

    def ajax_url(regex, view, kwargs=None, name=None):
        pattern = RegexPattern(regex, name=name, is_endpoint=True)
        return AjaxURLPattern(pattern, view, kwargs, name)

else:

    class AjaxURLPattern(URLPattern):
        def resolve(self, path):
            match = self.pattern.match(path)
            if match:
                new_path, args, kwargs = match

                name = kwargs.pop("name")
                if name not in registry:
                    return

                view = registry[name]
                return ResolverMatch(
                    view, args, kwargs, self.pattern.name, route=str(self.pattern)
                )

    def ajax_url(regex, view, kwargs=None, name=None):
        pattern = RegexPattern(regex, name=name, is_endpoint=True)
        return AjaxURLPattern(pattern, view, kwargs, name)


app_name = "ajax_views"
urlpatterns = [
    ajax_url(r"(?P<name>[-\w.]+)/", router, name="router"),
]
